#include "gl_window.h"
#include <stdio.h>
#include <stdlib.h>

gl_window :: gl_window(int x, int y, int h, int w, const char* label) :
Fl_Gl_Window(x, y, h, w, label)
{ 
	/* create the window */
	glutCreateWindow("");

	/* set double buffer, rga, depth sorted */
	mode(FL_DOUBLE | FL_RGB | FL_DEPTH);
	
	/* init clip planes */
	near_clip = 0.1;
	far_clip = 100.0;
	asp_ratio = h / w;

}

gl_window :: ~gl_window(void)
{
	// do nothing yet
}

int gl_window :: handle(int e)
{

	// if we have a mouse button push
	if(e == FL_PUSH)
	{
		// check which button
		if(Fl :: event_state() == FL_BUTTON1)
		{
			redraw();
			return 1;
		}
	}
	// maybe it was the keyboard
	else
	if(e == FL_KEYBOARD)
	{
		// was it the a key?
		if(Fl :: event_key() == 'a') 
		{
			redraw();
			return 1;
		}
	}
	
	// must have been something we don't care about
	
	return 0;
}

void gl_window :: draw(void)
{

	// give ourselves focus
	Fl :: focus(this);
	
	/* init hardware rendering */

	/* set the background color */
	glClearColor(0, 0, 0, 1);

	/* reset the buffer bits */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* drop to projection mode */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* set up the viewing frustum */
	glFrustum(-asp_ratio * .05, asp_ratio * .05, -.05, .05,
	near_clip, far_clip);

	/* go back to world coordinates */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* move out */
	glTranslatef(0.0f, 0.0f, -5.0f);

	/* rotate object */
	glRotatef(30, 1, 0, 0);

	/* set the drawing color */
	glColor3f(.5, .6, .7);

	/* draw something */
	glBegin(GL_TRIANGLES);
		
		for(int i = 0; i < 3; i++)
			glVertex3f( random() / (float) RAND_MAX, 
						random() / (float) RAND_MAX,
						random() / (float) RAND_MAX);
		
	glEnd();
	
	/* swap buffers */
	glutSwapBuffers();

}
