#include "gl_window.h"

gl_window :: gl_window(int x, int y, int h, int w, const char* label) :
Fl_Gl_Window(x, y, h, w, label)
{ 
	/* create the window */
	glutCreateWindow("");

	/* set double buffer, rga, depth sorted */
	mode(FL_DOUBLE | FL_RGB | FL_DEPTH);
	
	/* init clip planes */
	near_clip = 0.1;
	far_clip = 15.0;
	asp_ratio = h / w;

	/* rotation increments */
	rotx = 0; roty = 30; rotz = 149;

	/* init texture info */
	for(int i = 0; i < 5; i++)
	{
		tex_width[i] = 0;
		tex_height[i] = 0;
		_texture[i] = NULL;
	}

	/* init camera position and orientation */
	cam_x = 0; cam_y = 0; cam_z = -2;
	eye_x = 0; eye_y = 0; cam_z = 5;

	fog_r = 1; fog_g = 1; fog_b = 1;

	loadPPM("tux.ppm", 1);
	loadPPM("bill.ppm", 2);
	loadPPM("gabe.ppm", 3);
	loadPPM("wood.ppm", 4);

	fog_density = .5;
}

gl_window :: ~gl_window(void)
{
	// do nothing yet
}

void gl_window :: rotate(void)
{
	rotx < 360 ? (rand() % 10 > 5 ? rotx++ : rotx += 2) : rotx -= 360;
	roty < 360 ? (rand() % 10 > 5 ? roty+= 3 : roty += 2) : roty -= 360;
	//rotz < 360 ? (rand() % 10 > 5 ? rotz++ : rotz += 2) : rotz -= 360;

	redraw();

}

int gl_window :: handle(int e)
{

	// if we have a mouse button push
	if(e == FL_PUSH)
	{
		// check which button
		if(Fl :: event_state() == FL_BUTTON1)
		{
			redraw();
			return 1;
		}
	}
	// maybe it was the keyboard
	else
	if(e == FL_KEYBOARD)
	{
		// was it the a key?
		if(Fl :: event_key() == 'a') cam_x-= .1;
		if(Fl :: event_key() == 's') cam_z+= .1;
		if(Fl :: event_key() == 'd') cam_x+= .1;
		if(Fl :: event_key() == 'w') cam_z-= .1;
		if(Fl :: event_key() == 'j') eye_x-= .1;
		if(Fl :: event_key() == 'k') eye_y-= .1;
		if(Fl :: event_key() == 'l') eye_x+= .1;
		if(Fl :: event_key() == 'i') eye_y+= .1;

		printf("%f %f %c\n", cam_x, cam_z, Fl :: event_key());
		fflush(stdout);
				
		redraw();
		return 1;
	}
	
	// must have been something we don't care about
	
	return 0;
}

void gl_window :: draw(void)
{
	// basic light
	static GLfloat light_pos[] = {0, 2, 5, 0};
	static GLfloat diffuse[] = {0, 0, 0, 1};
	static GLfloat specular[] = {1, 1, 1, .2};
	//static GLfloat mat_specular[] = {1, 1, 1, .5};
	//static GLfloat mat_diffuse[] = {0, 0, 0, .2};
	static GLfloat mat_specular[] = {1, 1, 1, 1};
	static GLfloat mat_diffuse[] = {0, 0, 0, 1};

	//static GLfloat fog_color[] = {0, .5, .8, .3};
	GLfloat fog_color[] = {fog_r, fog_g, fog_b, .3};
	
	// give ourselves focus
	Fl :: focus(this);
	
	/* give GL some hints */
	glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
	glHint(GL_FOG_HINT, GL_FASTEST);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	/* init hardware rendering */

	/* set the background color */
	//glClearColor(0, .5, .8, 1);
	glClearColor(fog_r, fog_g, fog_b, 1);

	/* reset the buffer bits */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* drop to projection mode */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* set up the viewing frustum */
	glFrustum(-asp_ratio * .05, asp_ratio * .05, -.05, .05,
	             near_clip, far_clip);

	/* set up the camera */
	gluLookAt(cam_x, cam_y, cam_z, eye_x, eye_y, eye_z, 0, 1, 0);

	/* go back to world coordinates */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* enable checks */
	glEnable(GL_DEPTH);
	glEnable(GL_DEPTH_TEST);

	// for transparency 
	glEnable(GL_BLEND);

	/* enable lighting */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// lighting properties
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
	
	// material properties
	glMaterialf(GL_FRONT, GL_SHININESS, .9);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);

	// Gouraud Shading
	glShadeModel(GL_SMOOTH);

	// enable fog
	glEnable(GL_FOG);

	// fog properties
	glFogi(GL_FOG_MODE, GL_EXP2);
	glFogfv(GL_FOG_COLOR, fog_color);
	glFogf(GL_FOG_DENSITY, fog_density);
	glFogf(GL_FOG_START, 1);
	glFogf(GL_FOG_END, 10.0);

	// transparency
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	/* move out */
	glTranslatef(0.0f, 0.0f, -5.0f);

	/* draw floor */
	glDisable(GL_BLEND);
	loadTex(4);
	glBegin(GL_QUADS);
		glNormal3f(0, 1, 0);
		glTexCoord2f(0, 0); glVertex3d(-5, -1, -4);
		glTexCoord2f(4, 4); glVertex3d(-4, -1, 4);
		glTexCoord2f(0, 4); glVertex3d(4, -1, 4);
		glTexCoord2f(4, 0); glVertex3d(5, -1, -4);
	glEnd();
	glEnable(GL_BLEND);

	glNewList(1, GL_COMPILE);
		drawBox(1);
	glEndList();
	
	glTranslatef(-2, -1, 0);
	
	for(int i = 0; i < 3; i++)
	{
		glTranslatef(i, .5, 0);

		for(int j = 0; j < 3; j++)
		{
			glPushMatrix();

				glTranslatef(j, 0, j * -.5);
				
				(j % 2 == 0) ? loadTex(3) : loadTex(2);
				
				glRotatef(roty, 0, 1, 0);
				glCallList(1);

				//glTranslatef(-j, 0, 0);

			glPopMatrix();
		}

		glTranslatef(-i, .5, 0);

	}

	/* swap buffers */
	glutSwapBuffers();

}
